/** @file dig_outputs.c
 
   Contains all functions to set digital outputs of the APCI-1696.
 
   @par CREATION  
   @author Krauth Julien
   @date   10.06.05
   
   @par VERSION
   @verbatim
   $LastChangedRevision:$
   $LastChangedDate:$
   @endverbatim   
   
   @par COMPILER
   @verbatim
   Compiler	: gcc
   Version	: gcc (GCC) 3.3.6 (Debian 1:3.3.6-5)   
   kernel	: 2.4.26
   @endverbatim  
   
   @par UPDATES
   @verbatim
   Date			:
   Author		: 
   Description	: 
   @endverbatim    
    
   @par LICENCE
   @verbatim
   Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
        
        ADDI-DATA GmbH 
        Dieselstrasse 3 
        D-77833 Ottersweier 
        Tel: +19(0)7223/9493-0 
        Fax: +49(0)7223/9493-92 
        http://www.addi-data-com 
        info@addi-data.com 
        
   This program is free software; you can redistribute it and/or modify it under 
   the terms of the GNU General Public License as published by the Free Software 
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
   or FITNESS FOR A PARTICULAR PURPOSE. 
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with 
   this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   You shoud also find the complete GPL in the COPYING file 
   accompanying this source code.
   @endverbatim   
 */
 
#include "apci1696-private.h"
#include <apci1696-kapi.h>

/**@def EXPORT_NO_SYMBOLS
 * Function in this file are not exported.
 */
EXPORT_NO_SYMBOLS;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,27)
#define __user 
#endif

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

/** Set all digital outputs of the APCI-1696 on.
 *
 * @param [in,out]	pdev	Pointer on the device structure.
 * @param [in]		cmd		IOCTL command. The command is <br>
 * 							CMD_APCI1696_SetAllDigitalOutputsOn when using the IOCTL funktion.
 * @param 			arg		Function argument. An array of 12 unsigned char that has contain ports states.<br>
 *                          Each index of the array is a port e.g.:<br>
 *                          PortsState [0]: Port 0, is composed of 8 channels. One channel can be 1 (high) or 0 (low).
 *                          PortsState [11]: Port 11, is composed of 8 channels. One channel can be 1 (high) or 0 (low).
 *
 * @retval 0		No error. 
 * @retval -EFAULT	Can't return parameters.
 */
int do_CMD_APCI1696_SetAllDigitalOutputsOn(struct pci_dev *pdev, unsigned int cmd, unsigned long arg)
{
	unsigned char PortsState[APCI1696_PORT_NUMBER]; /**< Ports states:<br>
                                     One port is composed of 8 channels.<br>
                                     One channel can be 1 (high) or 0 (low) */
	int ret = 0;
	unsigned long irqstate;

	if ( copy_from_user(PortsState,(unsigned char __user *)arg, sizeof(PortsState) ) )
		return -EFAULT;
		                 
	if (APCI1696_LOCK(pdev,&irqstate))
  		return -ERESTARTSYS;
	{
		ret = APCI1696_SetAllDigitalOutputsOn(pdev, PortsState);
	}      
	APCI1696_UNLOCK(pdev,irqstate); 
	
  	return ret;
}

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

/** Set all digital outputs of the APCI-1696 off.
 *
 * @param [in,out]	pdev	Pointer on the device structure.
 * @param [in]		cmd		IOCTL command. The command is <br>
 * 							CMD_APCI1696_SetAllDigitalOutputsOff when using the IOCTL funktion.
 * @param 			arg		Function argument. An array of 12 unsigned char that has contain ports states.<br>
 *                          Each index of the array is a port e.g.:<br>
 *                          PortsState [0]: Port 0, is composed of 8 channels. One channel can be 1 (high) or 0 (low).
 *                          PortsState [11]: Port 5, is composed of 8 channels. One channel can be 1 (high) or 0 (low).
 *
 * @retval 0		No error. 
 * @retval -EPERM	Output memory is not activated.
 * @retval -EFAULT	Can't return parameters.
 */
int do_CMD_APCI1696_SetAllDigitalOutputsOff(struct pci_dev *pdev, unsigned int cmd, unsigned long arg)
{
	unsigned char PortsState[APCI1696_PORT_NUMBER]; /**< Ports states:<br>
                                     One port is composed of 8 channels.<br>
                                     One channel can be 1 (high) or 0 (low) */
	int ret = 0;
	unsigned long irqstate;
	
	if ( copy_from_user(PortsState,(unsigned char __user *)arg, sizeof(PortsState) ) )
		return -EFAULT;
		                 
	if (APCI1696_LOCK(pdev,&irqstate))
  		return -ERESTARTSYS;
	{
		ret = APCI1696_SetAllDigitalOutputsOff(pdev, PortsState);
	}      
	APCI1696_UNLOCK(pdev,irqstate);
	
	return ret;
}

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

/** Set the digital output memory on.
 *
 * @param [in,out]	pdev	Pointer on the device structure.
 * @param [in]		cmd		IOCTL command. The command is <br>
 * 							CMD_APCI1696_SetDigitalOutputMemoryOn when using the IOCTL funktion.<br>
 * 							To call the function in kernel mode, cmd must be 0.
 * @param 			arg		Non arg, it can be NULL.
 *
 * @retval 0		No error.
 */
int do_CMD_APCI1696_SetDigitalOutputMemoryOn(struct pci_dev *pdev, unsigned int cmd, unsigned long arg)
{
	int ret;
	unsigned long irqstate;

	if (APCI1696_LOCK(pdev,&irqstate))
  		return -ERESTARTSYS;
	{
		ret = APCI1696_SetDigitalOutputMemoryOn(pdev);
	}      
	APCI1696_UNLOCK(pdev,irqstate);

	return ret;
}

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

/** Set the digital output memory off.
 *
 * @param [in,out]	pdev	Pointer on the device structure.
 * @param [in]		cmd		IOCTL command. The command is <br>
 * 							CMD_APCI1696_SetDigitalOutputMemoryOff when using the IOCTL funktion.<br>
 * 							To call the function in kernel mode, cmd must be 0.
 * @param 			arg		Non arg, it can be NULL.
 *
 * @retval 0		No error.
 */
int do_CMD_APCI1696_SetDigitalOutputMemoryOff(struct pci_dev *pdev, unsigned int cmd, unsigned long arg)
{
	int ret;
	unsigned long irqstate;

	if (APCI1696_LOCK(pdev,&irqstate))
  		return -ERESTARTSYS;
	{
		ret = APCI1696_SetDigitalOutputMemoryOff(pdev);
	}      
	APCI1696_UNLOCK(pdev,irqstate);

	return ret;

}

//------------------------------------------------------------------------------

/** Get all digital outputs states of the APCI-1696.
 *
 * @param [in,out]	pdev	Pointer on the device structure.
 * @param [in]		cmd		IOCTL command. The command is <br>
 * 							CMD_APCI1696_GetAllDigitalOutputsStates when using the IOCTL funktion.
 * @param 			arg		Function argument. An array of 12 unsigned char that contain ports states.<br>
 *                          Each index of the array is a port e.g.:<br>
 *                          PortsState [0]: Port 0, is composed of 8 channels. One channel can be 1 (high) or 0 (low).
 *                          PortsState [11]: Port 11, is composed of 8 channels. One channel can be 1 (high) or 0 (low).
 *
 * @retval 0		No error. 
 * @retval -EFAULT	Can't return parameters.
 */
int do_CMD_APCI1696_GetAllDigitalOutputsStates(struct pci_dev *pdev, unsigned int cmd, unsigned long arg)
{
	unsigned char PortsState[APCI1696_PORT_NUMBER]; /**< Ports states:<br>
                                     One port is composed of 8 channels.<br>
                                     One channel can be 1 (high) or 0 (low) */
	int ret = 0;
	unsigned long irqstate;

	if (APCI1696_LOCK(pdev,&irqstate))
  		return -ERESTARTSYS;
	{
		ret = APCI1696_GetAllDigitalOutputsStates(pdev, PortsState);
	}      
	APCI1696_UNLOCK(pdev,irqstate);

	if(ret)
		return ret;

	if ( copy_to_user((unsigned char __user *)arg, PortsState, sizeof(PortsState) ) )
		return -EFAULT;
       
      return 0;
}
	

