/** @file dig_inputs.c
 
   Contains all functions to read digital inputs of the APCI-1696.
 
   @par CREATION  
   @author Krauth Julien
   @date   10.06.05
   
   @par VERSION
   @verbatim
   $LastChangedRevision:$
   $LastChangedDate:$
   @endverbatim   
   
   @par COMPILER
   @verbatim
   Compiler	: gcc
   Version	: gcc (GCC) 3.3.6 (Debian 1:3.3.6-5)   
   kernel	: 2.4.26
   @endverbatim  
   
   @par UPDATES
   @verbatim
   Date			:
   Author		: 
   Description	: 
   @endverbatim    
    
   @par LICENCE
   @verbatim
   Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
        
        ADDI-DATA GmbH 
        Dieselstrasse 3 
        D-77833 Ottersweier 
        Tel: +19(0)7223/9493-0 
        Fax: +49(0)7223/9493-92 
        http://www.addi-data-com 
        info@addi-data.com 
        
   This program is free software; you can redistribute it and/or modify it under 
   the terms of the GNU General Public License as published by the Free Software 
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
   or FITNESS FOR A PARTICULAR PURPOSE. 
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with 
   this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   You shoud also find the complete GPL in the COPYING file 
   accompanying this source code.
   @endverbatim   
 */
 
#include "apci1696-private.h"
#include <apci1696-kapi.h>

/**@def EXPORT_NO_SYMBOLS
 * Function in this file are not exported.
 */
EXPORT_NO_SYMBOLS;

//------------------------------------------------------------------------------

/** Read all digital inputs of the APCI-1696.
 *
 * @param [in,out]	pdev	Pointer on the device structure.
 * @param 			arg		Function argument. An array of 12 unsigned char that will contain ports states.<br>
 *                          Each index of the array is a port e.g.:<br>
 *                          PortsSetting [0]: Port 0, is composed of 8 channels. One channel can be 1 (high) or 0 (low).
 *                          PortsSetting [5]: Port 5, is composed of 8 channels. One channel can be 1 (high) or 0 (low).
 *
 * @retval 0		No error.  
 */
int APCI1696_ReadAllDigitalInputs(struct pci_dev *pdev, unsigned char PortsState[APCI1696_PORT_NUMBER])
{
	/* Initialize the PortsState */
	memset (PortsState, 0, (sizeof (unsigned char) * 6));
    
    /* 0 to 3 */
    APCI1696_PRIVDATA(pdev)->RegisterDigitalInput.Register1.AllPorts = inl((GET_BAR0(pdev) + APCI1696_READREG_1));
    /* 4 to 5 */
    APCI1696_PRIVDATA(pdev)->RegisterDigitalInput.Register2.AllPorts = inl((GET_BAR0(pdev) + APCI1696_READREG_2));         
    /* 6 to 11 */
    APCI1696_PRIVDATA(pdev)->RegisterDigitalInput.Register3.AllPorts = inl((GET_BAR0(pdev) + APCI1696_READREG_3));

    /* Copy ports settings to return them to user */
    {
    	int i;
	    for (i=0; i<4; i++)
	    {
	    	PortsState [APCI1696_PORT0 + i] = APCI1696_PRIVDATA(pdev)->RegisterDigitalInput.Register1.SinglePort[i];         
	    	PortsState [APCI1696_PORT4 + i] = APCI1696_PRIVDATA(pdev)->RegisterDigitalInput.Register2.SinglePort[i];
	    	PortsState [APCI1696_PORT8 + i] = APCI1696_PRIVDATA(pdev)->RegisterDigitalInput.Register3.SinglePort[i];
	    }
    }
	return 0;
}
